// dccp_option.hpp
// ~~~~~~~~~~~~~~~
//
// Copyright (c) 2009 Alex Dubov (oakad at yahoo dot com)
//
// Distributed under the Boost Software License, Version 1.0. (See accompanying
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
//

#ifndef BOOST_ASIO_DCCP_BASE_HPP
#define BOOST_ASIO_DCCP_BASE_HPP

#if defined(_MSC_VER) && (_MSC_VER >= 1200)
# pragma once
#endif // defined(_MSC_VER) && (_MSC_VER >= 1200)

#include <boost/asio/detail/push_options.hpp>

#include "vector_socket_option.hpp"

namespace boost {
namespace asio {

class dccp_option
{
public:
#ifndef SOL_DCCP
#define SOL_DCCP 269
#endif

#ifndef SOL_DCCP
#define SOL_DCCP 269
#endif

#ifndef DCCP_SOCKOPT_SERVICE
#define DCCP_SOCKOPT_SERVICE 2
#endif

#if defined(GENERATING_DOCUMENTATION)
  typedef implementation_defined service;
#else
  typedef boost::asio::detail::socket_option::vector<
    SOL_DCCP, DCCP_SOCKOPT_SERVICE, unsigned int> service;
#endif

#ifndef DCCP_SOCKOPT_GET_CUR_MPS
#define DCCP_SOCKOPT_GET_CUR_MPS 5
#endif

#if defined(GENERATING_DOCUMENTATION)
  typedef implementation_defined cur_mps;
#else
  typedef boost::asio::detail::socket_option::integer<
    SOL_DCCP, DCCP_SOCKOPT_GET_CUR_MPS> cur_mps;
#endif

#ifndef DCCP_SOCKOPT_SEND_CSCOV
#define DCCP_SOCKOPT_SEND_CSCOV 10
#endif

#if defined(GENERATING_DOCUMENTATION)
  typedef implementation_defined send_cscov;
#else
  typedef boost::asio::detail::socket_option::integer<
    SOL_DCCP, DCCP_SOCKOPT_SEND_CSCOV> send_cscov;
#endif

#ifndef DCCP_SOCKOPT_RECV_CSCOV
#define DCCP_SOCKOPT_RECV_CSCOV 11
#endif

#if defined(GENERATING_DOCUMENTATION)
  typedef implementation_defined recv_cscov;
#else
  typedef boost::asio::detail::socket_option::integer<
    SOL_DCCP, DCCP_SOCKOPT_RECV_CSCOV> recv_cscov;
#endif

#if 0
#if defined(GENERATING_DOCUMENTATION)
  typedef implementation_defined cur_mps;
#else
  typedef boost::asio::detail::socket_option::tfrc_rx<
    SOL_DCCP, DCCP_SOCKOPT_CCID_RX_INFO> ccid_rx_info;
#endif

#if defined(GENERATING_DOCUMENTATION)
  typedef implementation_defined cur_mps;
#else
  typedef boost::asio::detail::socket_option::tfrc_tx<
    SOL_DCCP, DCCP_SOCKOPT_CCID_TX_INFO> ccid_tx_info;
#endif
#endif
};


} // namespace asio
} // namespace boost

#include <boost/asio/detail/pop_options.hpp>

#endif // BOOST_ASIO_DCCP_OPTIONS_HPP
