//
// vector_socket_option.hpp
// ~~~~~~~~~~~~~~~~~
//
// Copyright (c) 2009 Alex Dubov (oakad at yahoo dot com)
//
// Distributed under the Boost Software License, Version 1.0. (See accompanying
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
//

#ifndef BOOST_ASIO_VECTOR_SOCKET_OPTION_HPP
#define BOOST_ASIO_VECTOR_SOCKET_OPTION_HPP

#if defined(_MSC_VER) && (_MSC_VER >= 1200)
# pragma once
#endif // defined(_MSC_VER) && (_MSC_VER >= 1200)

#include <boost/asio/detail/push_options.hpp>

#include <boost/asio/detail/push_options.hpp>
#include <cstddef>
#include <stdexcept>
#include <boost/config.hpp>
#include <boost/asio/detail/pop_options.hpp>

#include <boost/asio/detail/socket_types.hpp>

namespace boost {
namespace asio {
namespace detail {
namespace socket_option {

template <int Level, int Name, typename T>
class vector
{
public:
  // Default constructor.
  vector()
    : value_(1, 0)
  {
  }

  // Construct with a specific option value.
  explicit vector(const std::vector<T> &v)
    : value_(v)
  {
    if (value_.empty())
      value_.insert(value_.end(), 1, 0);
  }

  explicit vector(T v)
    : value_(1, v)
  {
  }

  // Get the level of the socket option.
  template <typename Protocol>
  int level(const Protocol&) const
  {
    return Level;
  }

  // Get the name of the socket option.
  template <typename Protocol>
  int name(const Protocol&) const
  {
    return Name;
  }

  // Get reference to the vector data.
  std::vector<T>& value()
  {
    return value_;
  }

  const std::vector<T>& value() const
  {
    return value_;
  }

  T& operator[](std::size_t n)
  {
    return value_[n];
  }

  const T& operator[](std::size_t n) const
  {
    return value_[n];
  }

  // Get the address of the vector data.
  template <typename Protocol>
  T* data(const Protocol&)
  {
    return &value_.front();
  }

  // Get the address of the vector data.
  template <typename Protocol>
  const T* data(const Protocol&) const
  {
    return &value_.front();
  }

  // Get the size of the vector data.
  template <typename Protocol>
  std::size_t size(const Protocol&) const
  {
    return value_.size() * sizeof(T);
  }

 // Set the size of the vector data.
  template <typename Protocol>
  void resize(const Protocol&, std::size_t s)
  {
    if (s && !(s % sizeof(T))) {
      value_.resize(s / sizeof(T));
    } else {
      throw std::length_error("vector socket option resize");
    }
  }

private:
  std::vector<T> value_;
};

} // namespace socket_option
} // namespace detail
} // namespace asio
} // namespace boost

#include <boost/asio/detail/pop_options.hpp>

#endif // BOOST_ASIO_VECTOR_SOCKET_OPTION_HPP
